#!/usr/bin/perl

my @Id    = split / /,'$Id: nodelist2dns.pl,v 1.7 2010/12/20 02:46:50 stas Exp stas $';
my $ver   = $Id[2];
my $banner= "nodelist2dns v.$ver - A perl script to convert nodelist to DNS (bind) zone file";
# (c) Copyleft 2009-2010 Stas Degteff 2:5080/102@fidonet
# (c) Copyleft 2010 Robert James Clay 1:120/544@fidonet
# Distributes on GNU GPL v2, see http://www.gnu.org

# Features:
# - make "A" and "CNAME" records like "fNNNN.nMMMM.zZ" for nodes Z:MMMM/NNNN
# - make "A" and "CNAME" records like "f0.nMMMM.zZ" for host nodes Z:MMMM/0
# - IP address and domain name is taken from "BBS name" field of the nodelist
#   for nodes with IBN flag
# - parse INA flag
# - ignore "Down" and "Hold" nodes

# ToDo:
# - parse parameter of Ixx flags
# - parse flags "DOM", "DO4", "DO3", "DO2"

# What it's works.
# Scans nodelist:
# - skip any nodes without flag "IBN";
# - for lines begins with "Zone,", "Region,", "Host,", "Hub,", "Pvt,", "," check ",INA:"
#   and match address after this flag;
# - if "INA" not presented then math IP (numberic) address or fully-qualified
#   domain name in field 3 ("station/BBS name").

# Changelog.
# 1.0 
# * first public release
# 1.1 
# + parse "INA" flag
# ! parse "Region" lines (bugfix)
# ! fixed ip address detection (regexp bugfix)
# + display warnings if incorrect nodelist line is found
# + if domain name or IP address isn't specified then generate default domain
#   name (using template fXXX.nYYY.zZ.fidonet.net. "fidonet.net" is specified
#   in configuration section and may be changed)
# + multiple addresses: first taken from "INA" flag, second taken from BBS name
#   field
# 1.2 
# ! Write CNAME only if A records not wroted for host.
# 1.3 
# + Add $TTL directive into zone file
# 1.4 
# + Skip empty (last) line of nodelist
# + Improve disgnostic
# 1.5 
# + Skip last line if one contains Ctrl-Z only
# 1.6
# + Debug mode
# ! Allow four-char TLD  (like "info")
# + Parse phone number string
# ! Fix domain name detection for station name field
# + Parse IBN flag
# 1.7
# + Use DNS::ZoneParse Perl Module to generate the new bind zone file
# + Pragmas - use warnings & use strict
#

use DNS::ZoneParse;

# configuration
#
my $TTL    =7200;                       # TTL for the DNS zone SOA record
my $server ="srv.grumbler.org";         # DNS host
my $person ="hostmaster.grumbler.org";  # responsible person (email address)
my $minttl =3600;                       # minimum zone's TTL
my $refresh=86400;                      # zone's refresh period
my $retry  =3600;                       # zone's retry period
my $expire =604800;                     # zone's expire time
my $default_root_domain="fidonet.net";  # domain for constucts hostname if
                                        # IP address/domain isn't specified in
                                        # nodelist line.
#
# configuration end
#
print "$banner\n";

if($#ARGV<0 || $#ARGV>2){

  print <<USAGE;

USAGE: nodelist2dns [--debug] nodelist.file dns.zone.file

USAGE
  exit;
}

$person =~ s/@/./;  # if responsible person email specified like name@email.domain

my $debug;
if( $ARGV[0] eq "--debug" ){
  $debug=1;
  shift @ARGV;
}
my ($nodelist,$zonefile)=@ARGV;

if( -f $nodelist && -r $nodelist){
  open NODELIST, $nodelist || die "Can't open $nodelist: $!\n";
  print "Parse nodelist $nodelist\n";
}else{ die "$nodelist is not readable\n"; }
open DNSZONE, ">$zonefile" || die "Can't create $zonefile: $!\n";


my $zone_contents = "\@		IN	NS	$server.";

my $dns_file = DNS::ZoneParse->new( \$zone_contents, $default_root_domain );
my $a_records = $dns_file->a(); 
my $cname_records = $dns_file->cname();
my $soa_record = $dns_file->soa();

&process;

my $newfile = $dns_file->output();

print DNSZONE $newfile;

print "File '$zonefile' is generated\n";

close NODELIST;
close DNSZONE;
exit 0;

sub process{

my $line = <NODELIST>;

  my $sn;

# ;A FidoNet Nodelist for Friday, September 2, 2005 -- Day number 245 : 31474
# $sn="20052450";
if( $line =~ /;A FidoNet Nodelist for [\w\s,]+, (\d\d\d\d) -- Day number (\d+) /){
  my $year=$1;
  my $day=$2;
  $day="0".$day if($day<10);
  $day="0".$day if($day<100);
  $sn=$year.$day."0";
}else{
  die "Wrong nodelist!\n";
}
  
  ${$soa_record}{'serial'} = $sn;
  ${$soa_record}{'origin'} = $default_root_domain;
  ${$soa_record}{'primary'} = $server;
  ${$soa_record}{'email'} = $person;
  ${$soa_record}{'refresh'} = $refresh;
  ${$soa_record}{'retry'} = $retry;
  ${$soa_record}{'ttl'} = $TTL;
  ${$soa_record}{'minimumTTL'} = $minttl;
  ${$soa_record}{'expire'} = $expire;


  my ($zone,$net,$node);
  my $linecount=1;
  while(<NODELIST>){
    $linecount++;
    next if(/^;/);
    next if( /^[\r\n\x1a]*$/ and eof(NODELIST) );
    if(/^[^\w,]/) { print "Warning! Illegal line ${linecount}:\n", $_;};

    print STDERR "Line \"$_\"\n" if( $debug );
    if(/^Zone,(\d+),/){
      $zone=$1;
      $net=$1;
      $node=0;
      print STDERR "Zone $zone\n" if( $debug );
    }
    elsif(/^(Region|Host),(\d+),/){ # net/independent number
      $net=$2;
      $node=0;
      print STDERR "Net(Region/Host) $zone:$net\n" if( $debug );
    }
    elsif(/^(Down|Hold),/){      # skip nodes with Down/Hold status.
      print STDERR "Down/Hold,skip\n" if( $debug );
      next
    }
    elsif(/^(Hub|Pvt|),(\d+),/){ # node number
      $node=$2;
      print STDERR "Node: $zone:$net/$node\n" if( $debug );
    }
    else{
      print STDERR "...skip\n" if( $debug );
      next;
    }

    s/[\r\n]$//;
    if(!/,IBN(:.*)?(,.*)?$/){ # see binkp nodes only
      print STDERR "Not IBN, skip\n" if( $debug );
      next
    }
    my @host;

    if( /,INA:([\w][\w\.-]+\.\w{2,4}|\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})(,.*|:|[\r\n]+)?$/){
      # INA flag
      push @host, lc($1);
      print STDERR " From INA: $1\n" if( $debug );
    }
    if( /,IBN:([\w][\w\.-]+\.\w{2,4}|\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})(,.*|:|[\r\n]+)?$/){
      # IBN flag
      push @host, lc($1);
      print STDERR " From IBN: $1\n" if( $debug );
    }
    if( /^\w+?,\d+,([^,]+,){3}000-([0-2]\d{1,2})-([0-2]\d{1,2})-([0-2]\d{1,2})-([0-2]\d{1,2}),/ ){
      # Phone number field
      if( $5>0 && $5<255 && $2>0 && $2<255 && $3>0 && $3<255 && $4>0 && $4<255 ){
        push @host,"$2.$3.$4.$5"; 
        print STDERR " From phone number filed: $2.$3.$4.$5\n" if( $debug );
      }
    }
    if( /^(\w+)?,\d+,([\w][\w\.-]+\.\w{2,4}|\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}),/ ){
      # station name field
      my $lc2 = lc($2);
      push @host, $lc2;
      print STDERR " From station name filed: $lc2\n" if( $debug );
    }
    if($#host==-1){
      push (@$cname_records, { name => "f$node.n$net.z$zone", class => 'IN',
                                host => "f$node.n$net.z$zone.$default_root_domain.", ttl => '' });
      print STDERR " Use default from fidonet.net\n" if( $debug );
      next;
    }

    {
      my $host;
      my $a=0;
      my @ip = sort( grep( /^(\d+\.){3}\d+$/, @host ) );
      while( $host = shift(@ip) ) {
        if( $host ne $ip[0] ){
          push (@$a_records, { name => "f$node.n$net.z$zone", class => 'IN',
                                host => $host, ttl => '' });
          print STDERR " Use $host\n" if( $debug );
          $a++;
        }else{ print STDERR " Duplicated $host\n" if( $debug ); }
      }
      if( !$a ){
        my @hostnames = grep( !/^(\d+\.){3}\d+$/, @host );
        foreach $host (@hostnames) {
          push (@$cname_records, { name => "f$node.n$net.z$zone", class => 'IN',
                                host => "$host.", ttl => '' });
          print STDERR " Use $host\n" if( $debug );
          last;
        }
      }
    }

  }

}

__END__
